@@ -1,4 +1,5 @@
 package CPANPLUS::Internals::Source;
+use deprecate;
 
 use strict;
 
@@ -18,6 +19,9 @@ use Params::Check               qw[check];
 use Module::Load::Conditional   qw[can_load];
 use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';
 
+use vars qw[$VERSION];
+$VERSION = "0.9135";
+
 $Params::Check::VERBOSE = 1;
 
 ### list of methods the parent class must implement
